use super::Error;
use std::{collections::HashMap, os::unix::net::UnixStream, path::PathBuf};

pub fn get_socket() -> Result<UnixStream, Error> {
    let socket_vars = std::env::vars()
        .filter(|(key, _)| ["XDG_RUNTIME_DIR", "KEYFORKD_SOCKET_PATH"].contains(&key.as_str()))
        .collect::<HashMap<String, String>>();
    let mut socket_path: PathBuf;
    #[allow(clippy::single_match_else)]
    match socket_vars.get("KEYFORKD_SOCKET_PATH") {
        Some(occupied) => {
            socket_path = PathBuf::from(occupied);
        }
        None => {
            socket_path = PathBuf::from(
                socket_vars
                    .get("XDG_RUNTIME_DIR")
                    .ok_or(Error::EnvVarsNotFound)?,
            );
            socket_path.extend(["keyforkd", "keyforkd.sock"]);
        }
    }
    UnixStream::connect(&socket_path).map_err(|e| Error::Connect(e, socket_path))
}
